"""This product is a monkey-patch aimed at making upgrading SilvaDocument
a bit more easier on me.  Any special modifications I make to SilvaDocument, like adding a kupu_editable function, will be placed in this monkeypatch.
"""
#silva
from Products.SilvaDocument.Document import Document
from Products.Silva import SilvaPermissions
from Products.Silva.ExtensionRegistry import extensionRegistry
from Products.Silva.fssite import registerDirectory

#local
import install, CustomEditorSupport
from TypographicalService import TypographicalService, manage_addTypographicalServiceForm, manage_addTypographicalService

def initialize(context):
    extensionRegistry.register('SilvaDocumentPatches',
                               'Patches for SilvaDocument', context,
                               [], install,
                               depends_on='SilvaDocument')
    context.registerClass(CustomEditorSupport.CustomEditorSupport,
                          constructors = (CustomEditorSupport.manage_addCustomEditorSupport, ),
                          )
    context.registerClass(TypographicalService,
                          constructors = (manage_addTypographicalServiceForm,
                                          manage_addTypographicalService),
                          icon='www/editor_service.gif'
                          )
    registerDirectory('widgets', globals())
    
    

def kupu_editable(self):
    """return true if this document doesn't contain elements which prevent
    this doc. from being editable in kupu"""
    editable = 1
    if not self._non_kupu_elements:
        return editable
    node = self.get_previewable().content.documentElement
    node_queue = [node]
    while node_queue:
        elem = node_queue.pop(0)
        if elem.nodeName in self._non_kupu_elements:
            editable = 0
            break
        elif (elem.hasChildNodes()):
            L = []
            for node in elem.childNodes:
                if node.nodeType == node.ELEMENT_NODE:
                    L.append(node)
            node_queue[:0] = L
    return editable
Document._non_kupu_elements = []
Document.kupu_editable = kupu_editable


def is_cacheable(self):
    """Return true if this document is cacheable.
    That means the document contains no dynamic elements like
    code, datasource or toc.
    """
    viewable = self.get_viewable()
    if viewable is None:
        return 0

    if not self._non_cacheable_elements:
        return 1

    # It should suffice to test the children of the root element only,
    # since currently the only non-cacheable elements are root elements
    for node in viewable.content.documentElement.childNodes:
        node_name = node.nodeName
        if node_name in self._non_cacheable_elements:
            return 0
        # FIXME: how can we make this more generic as it is very
        # specific now..?
        if node_name == 'source':
            is_cacheable = externalsource.isSourceCacheable(self.aq_inner, node)
            if not is_cacheable:
                return 0
    return 1
Document._non_cacheable_elements = ['toc', 'code',]
Document.is_cacheable = is_cacheable
