#silva
from Products.Silva.install import add_fss_directory_view
from Products.SilvaDocument.EditorSupportNested import EditorSupport,manage_addEditorSupport
from Products.SilvaDocument import externalsource
from Products.SilvaDocument.Document import Document

#local
import CustomEditorSupport
from CustomEditorSupport import CustomEditorSupport as CES
from TypographicalService import TypographicalService,manage_addTypographicalService

def is_installed(root):
    if hasattr(root,'service_editorsupport') and \
           root.service_editorsupport.meta_type==CES.meta_type:
        return 1
    return 0

def install(root):
    #setup CustomEditorSupport
    if hasattr(root.aq_explicit,'service_editorsupport') and \
       root.service_editorsupport.meta_type == EditorSupport.meta_type:
        root.manage_delObjects(['service_editorsupport'])
        CustomEditorSupport.manage_addCustomEditorSupport(root)
    if not hasattr(root.aq_explicit,'service_typo_chars'):
        manage_addTypographicalService(root,'service_typo_chars')
    installXMLWidgets(root)

def uninstall(root):
    root.manage_delObjects(['service_editorsupport','service_typo_chars'])
    manage_addEditorSupport(root)
    uninstallXMLWidgets(root)

def installXMLWidgets(root):
    add_fss_directory_view(root, 'service_sdp_widgets',
                           __file__, 'widgets')
    root.service_editor.addWidgetRoot('service_sdp_widgets', depends_on=('SilvaDocument',) )

    #add conditional services
    manage_addWidgetRegistry = root.manage_addProduct['XMLWidgets'].manage_addWidgetRegistry
    full_widgs = ('service_conditional_',)
    for name in ('viewer','previewer','editor'):
        for wr in full_widgs:
            manage_addWidgetRegistry(wr+name)
    
    registerDocEditor(root)
    registerDocPreviewer(root)
    registerDocViewer(root)
    registerSubEditor(root)
    registerSubPreviewer(root)
    registerSubViewer(root)

    registerConditionalEditor(root)
    registerConditionalViewer(root)
    registerConditionalPreviewer(root)

    registerTableEditor(root)
    registerTableViewer(root)

    #since these xml widgets make this document incompatible with kupu, add those
    #elements to the list...
    Document._non_kupu_elements.extend(['conditional','hr','custom_row_heading'])

    #also, we want to make this document uncacheable for some of the above:
    Document._non_cacheable_elements.append('conditional')
    
#sorts widget list alphabetically, but keeps 'p' at the top
def sort_widget_list(list, display_map):
    sort_list = []
    for l in list:
        dn = display_map.get(l,'')
        sort_list.append((dn,l))
    sort_list.sort()
    new_list = map(lambda x:x[1], sort_list)
    if 'p' in new_list:
        new_list.remove('p')
        new_list = ['p'] + new_list
    return new_list

#capitalizes display names
def fixDisplayNames(wr):
    map = wr._display_map
    for w in map.keys():
        wr.setDisplayName(w, map[w].capitalize())

doc_element_base = ['element','doc_elements']
sdp_element_base = ['service_sdp_widgets','element','sdp']
sdp_top_base     = ['service_sdp_widgets','top','sdp']

doc_elist = {
    'p': 'Paragraph',
    'heading': 'Heading',
    'pre': 'Preformatted',
    'list': 'List',
    'dlist': 'Definition list',
    'toc': 'Table of Contents',
    'image': 'Image',
    'cite': 'Citation',
    }
if externalsource.AVAILABLE:
    doc_elist['source'] =  'External Source'

nested_doc_elist = {
    'table': 'Table',
    'nlist': 'Complex list'
    }

doc_sub_elist = {
    'p': 'Paragraph',
    'heading': 'Heading',
    'pre': 'Preformatted',
    'list': 'List',
    'dlist': 'Definition list',
    'image': 'Image',
    'nlist': 'Complex list',
    'cite': 'Citation',    
    }

custom_widgets = { 'hr' : 'Horizontal Rule' }

###plug custom widgets into document editor####
def registerDocEditor(root):
    wr = root.service_doc_editor

    name = 'conditional'
    wr.addWidget(name, tuple(sdp_element_base+[name,'mode_normal']))
    wr.setDisplayName(name, 'Conditional Display')
    
    for name in custom_widgets.keys():
      wr.addWidget(name, tuple(sdp_element_base + [name,'mode_normal']))
      wr.setDisplayName(name, custom_widgets[name])
    fixDisplayNames(wr)
    cur_widgets = wr.getAllowed('doc')    
    cur_widgets = (cur_widgets + ['conditional'] + custom_widgets.keys())
    cur_widgets = sort_widget_list(cur_widgets, wr._display_map)    
    wr.setAllowed('doc', cur_widgets)
    
###plug custom widgets into document previewer####
def registerDocViewer(root):
    wr = root.service_doc_previewer
    for name in custom_widgets.keys() + ['conditional']:
        wr.addWidget(name,tuple(sdp_element_base + [name,'mode_preview']))
        
###plug custom widgets into document viewer####
def registerDocPreviewer(root):
    wr = root.service_doc_viewer
    for name in custom_widgets.keys() + ['conditional']:
        wr.addWidget(name,tuple(sdp_element_base + [name,'mode_view']))

def registerSubEditor(root):
    wr = root.service_sub_editor
    #conditional should be in this list, but it doesn't render correctly...
    for name in ('hr','conditional'):
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_normal']))
    #subs should be able to have tables
    wr.addWidget('table', tuple(doc_element_base + ['table','mode_normal']))
    wr.setDisplayName('table','Table')
    wr.setDisplayName('hr','Horizontal Rule')
    wr.setDisplayName('conditional','Conditional Display')
    if externalsource.AVAILABLE: #allow sources in subs
        wr.addWidget('source', (
            'service_widgets', 'element', 'doc_elements', 'source', 'mode_normal'))
        wr.setDisplayName('source', 'External source')
        allowed = wr.getAllowed('field')
        allowed.append('source')
        wr.setAllowed('field', allowed)

    cur_widgets = wr.getAllowed('field') +  ['hr','table','conditional',]

    cur_widgets = sort_widget_list(cur_widgets, wr._display_map)
    wr.setAllowed('field',(cur_widgets))
    cur_widgets = wr.getAllowed('li') + ['table']
    wr.setAllowed('li',(cur_widgets))
   
def registerSubPreviewer(root):
    wr = root.service_sub_previewer
    for name in ('hr','conditional'):
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_preview']))    
    wr.addWidget('table', tuple(doc_element_base + ['table','mode_view']))
    if externalsource.AVAILABLE:
        wr.addWidget('source', (
            'service_widgets', 'element', 'doc_elements', 'source', 'mode_preview'))

def registerSubViewer(root):
    wr = root.service_sub_viewer
    for name in ('hr','conditional'):
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_view']))    
    wr.addWidget('table', tuple(doc_element_base + ['table','mode_view']))
    if externalsource.AVAILABLE:
        wr.addWidget('source', (
            'service_widgets', 'element', 'doc_elements', 'source', 'mode_view'))
        
def registerConditionalViewer(root):
    wr = root.service_conditional_viewer
    wr.addWidget('conditional', tuple(sdp_top_base + ['conditional','mode_view']))
    for name in doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_view']))
    #XXX this needs to be updated in Tropos, so it uses the tropos-compat. widgets.
    for name in nested_doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_view']))
    for name in custom_widgets.keys():
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_view']))
        
def registerConditionalPreviewer(root):    
    wr = root.service_conditional_previewer
    wr.addWidget('conditional', tuple(sdp_top_base + ['conditional','mode_preview']))
    for name in doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_view']))
    for name in nested_doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_view']))
    for name in custom_widgets.keys():
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_view']))

def registerConditionalEditor(root):
    wr = root.service_conditional_editor
    wr.addWidget('conditional', tuple(sdp_top_base + ['conditional','mode_normal']))    
    for name in doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_normal']))
        wr.setDisplayName(name, doc_elist[name])
    for name in nested_doc_elist.keys():
        wr.addWidget(name, tuple(doc_element_base + [name,'mode_normal']))
        wr.setDisplayName(name, nested_doc_elist[name])
    for name in custom_widgets.keys():
        wr.addWidget(name, tuple(sdp_element_base + [name,'mode_normal']))
        wr.setDisplayName(name, custom_widgets[name])
    allowed = custom_widgets.keys() + doc_elist.keys() + nested_doc_elist.keys()
    allowed = sort_widget_list(allowed, wr._display_map)
    wr.setAllowed('conditional',allowed)

#XXX for some reason, custom_row_heading MUST be placed in SilvaDocument/widgets area...
#XXX also, top/table/mode_view/render.py needs modifications to support custom_row_heading
def registerTableEditor(root):
    wr = root.service_table_editor
    wr.addWidget('custom_row_heading', tuple(['element','table_elements', 'custom_row_heading', 'mode_normal']))
    wr.setDisplayName('custom_row_heading', 'Custom row heading')
    wr.setAllowed('table', ['row','row_heading','custom_row_heading'])
    
def registerTableViewer(root):
    wr = root.service_table_viewer
    wr.addWidget('custom_row_heading', tuple(['element','table_elements','custom_row_heading', 'mode_view']))

def uninstallXMLWidgets(root):
    for n in ['service_sdp_widgets',
              'service_conditional_viewer', 'service_conditional_editor',
              'service_conditional_previewer']:
        try:
            root.manage_delObjects([n])
        except:
            pass
    ed = root.service_doc_editor
    cur = ed.getAllowed('doc')
    for n in ['hr','conditional']:
        if n in cur:
            cur.remove(n)
        if n in ed._node_map:
            del ed._node_map[n]
    ed.setAllowed('doc',cur)
    
    ed = root.service_sub_editor
    cur = ed.getAllowed('field')
    for n in 'hr','table','conditional':
        if n in cur:
            cur.remove(n)
        if n in ed._node_map:
            del ed._node_map[n]
    ed.setAllowed('field',cur)

    ed = root.service_table_editor
    cur = ed.getAllowed('table')
    for n in ('custom_row_heading',):
        if n in cur:
            cur.remove(n)
        if n in ed._node_map:
            del ed._node_map[n]
    ed.setAllowed('field',cur)
