from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

from Products.Silva import SilvaPermissions
from Products.Silva import mangle

from Products.SilvaDocument import mixedcontentsupport as mcs
from customsilvasubset import PARAGRAPH_SUBSET,HEADING_SUBSET

from types import UnicodeType

from zLOG import LOG,INFO

class ParagraphSupport(mcs.ParagraphSupport):
    '''Overloads SilvaDocument.ParagraphSupport to provide support
    for <label> and class,style attributes in markup'''

    security = ClassSecurityInfo()

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'parse')
    def parse(self, inputstr):
        """ Parse input string to a DOM
        """
        if not isinstance(inputstr,UnicodeType):
            inputstr = unicode(inputstr, 'UTF-8')
        node = self._node
        # remove all old subnodes of node
        while node.hasChildNodes():
            node.removeChild(node.firstChild)
        PARAGRAPH_SUBSET.filteredParse(inputstr, node)

    def _renderHtmlHelper(self, node, view_type):
        result = []
        for child in node.childNodes:
            if child.nodeType == child.TEXT_NODE:
                result.append(mangle.entities(child.nodeValue))
                continue
            if child.nodeType != child.ELEMENT_NODE:
                continue
            if child.nodeName == 'strong':
                result.append('<strong')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</strong>')
            elif child.nodeName == 'em':
                result.append('<em')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</em>')
            elif child.nodeName == 'super':
                result.append('<sup')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')                
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</sup>')
            elif child.nodeName == 'sub':
                result.append('<sub')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</sub>')
            elif child.nodeName == 'span':
                result.append('<span')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</span>')
            elif child.nodeName == 'abbr':
                result.append('<abbr')
                if child.getAttribute('title'):
                    result.append(
                        ' title="%s"' % child.getAttribute('title'))
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</abbr>')
            elif child.nodeName == 'acronym':
                result.append('<acronym')
                if child.getAttribute('title'):
                    result.append(
                        ' title="%s"' % child.getAttribute('title'))
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</acronym>')
            elif child.nodeName == 'link':
                path = child.getAttribute('url')
                url = self._linkHelper(node, path,view_type)
                result.append('<a href="%s"' %  mangle.entities(url))
                if child.hasAttribute('target'):
                    target = child.getAttribute('target')
                    if target == '': target = '_blank'
                    result.append(
                        ' target="%s"' % mangle.entities(target))
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</a>')
            elif child.nodeName == 'label': #XXXXXXXXXXXXX#
                lfor = child.getAttribute('for')
                result.append('<label for="%s"'%lfor)
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</label>')
            elif child.nodeName == 'underline':
                result.append('<u')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</u>')
            elif child.nodeName == 'index':
                index = mangle.entities(child.getAttribute('name'))
                result.append('<a class="index-element" name="%s"' % index)
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))            
                # in nested lists and tables we cannot rely on the
                # correct view_type passed in, so check for edit_mode
                # from the request
		result.append('>')
                edit_mode = node.REQUEST.get('edit_mode', 0)
                if edit_mode:
                    result.append('[[%s]]' % index)
                result.append('</a>')
            elif child.nodeName == 'br':
                result.append('<br />')
            else:
                result.append('<span class="error">%s</span> ' % 
                    self._renderHtmlHelper(child, view_type))
        return ''.join(result)

    def _renderEditableHelper(self, node):
        """Render textual content as editable text.
        """
        result = []
        for child in node.childNodes:
            if child.nodeType == child.TEXT_NODE:
                result.append(child.nodeValue)
                continue
            if child.nodeType != child.ELEMENT_NODE:
                continue
            if child.nodeName == 'strong':
                result.append('<b')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</b>')
            elif child.nodeName == 'em':
                result.append('<i')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</i>')
            elif child.nodeName == 'super':
                result.append('<sup')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</sup>')
            elif child.nodeName == 'sub':
                result.append('<sub')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</sub>')
            elif child.nodeName == 'span':
                result.append('<span')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</span>')
            elif child.nodeName == 'abbr':
                if child.getAttribute('title'):
                    result.append(
                        '<abbr title="%s"' % child.getAttribute('title'))
                else:
                    result.append('<abbr')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</abbr>')
            elif child.nodeName == 'link':
                url = mangle.entities(child.getAttribute('url'))
                url_comparator = mangle.entities(
                    self._linkHelper(node, child.getAttribute('url')))
                if child.hasAttribute('target'):
                    target = child.getAttribute('target')
                    if target == '': target = '_blank'
                else:
                    target = mangle.entities(child.getAttribute('target'))
                linktext = self._renderEditableHelper(child)
                if (not target and linktext == url_comparator):
                    # XXX what is this?
                    result.append(url)
                else:
                    if target:
                        tag = '<a href="%s" target="%s"' % (url, target)
                    else:
                        tag = '<a href="%s"' % url
                        
                    if child.getAttribute('class'):
                        tag += ' class="%s"'%child.getAttribute('class')
                    if child.getAttribute('style'):
                        tag += ' style="%s"'%child.getAttribute('style')
                    tag += '>'
                    result.append(tag)
                    result.append(linktext)
                    result.append('</a>')
            elif child.nodeName == 'label': #XXXXXXXXXXXXX#
                lfor = child.getAttribute('for')
                result.append('<label for="%s"'%lfor)
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</label>')
            elif child.nodeName == 'underline':
                result.append('<u')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</u>')
            elif child.nodeName == 'index':
                result.append('<index')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(mangle.entities(child.getAttribute('name')))
                result.append('</index>')
            elif child.nodeName == 'acronym':
                if child.getAttribute('title'):
                    result.append(
                        '<acronym title="%s"' % child.getAttribute('title'))
                else:
                    result.append('<acronym')
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</acronym>')
            elif child.nodeName == 'abbr':
                result.append('<abbr')
                if child.getAttribute('title'):
                    result.append(
                        ' title="%s"' % child.getAttribute('title'))
                if child.getAttribute('class'):
                    result.append(' class="%s"'%child.getAttribute('class'))
                if child.getAttribute('style'):
                    result.append(' style="%s"'%child.getAttribute('style'))
                result.append('>')
                result.append(self._renderEditableHelper(child))
                result.append('</abbr>')
            elif child.nodeName == 'br':
                # \n is already in XML (or not)
                result.append('<br>')
            else:
                result.append(
                    'ERROR %s ERROR ' % self._renderEditableHelper(child))
        return ''.join(result)

    def _linkHelper(self, context, path,view_type='public'):
        if path == '':
            return path
        # If it is a url already, return it:
        if mcs._url_match.match(path):
            return path #don't do preview mode here
	preview_mode = ''
        if context.REQUEST.get('view_mode','public') == 'preview':
            view_type == 'preview'
            preview_mode = '/preview_html'
        # Is it a query or anchor fragment? If so, return it
        if path[0] in ['?', '#']:
            return preview_mode + path
        
        # It is not an URL, query or anchor, so treat it as a path.
        # If it is a relative path, treat is as such:
        if not path.startswith('/'):
            container = context.get_container()
            return container.absolute_url() + '/' + path + preview_mode
        # If it is an absolute path, try to traverse it to
        # a Zope/Silva object and get the URL for that.
        splitpath = [p.encode('ascii','ignore') for p in path.split('/') ]
        obj = context.restrictedTraverse(splitpath, None)
        if obj is None:
            # Was not found, maybe the link is broken, but maybe it's just 
            # due to virtual hosting situations or whatever.
            return path
        if hasattr(obj.aq_base, 'absolute_url'):
            # There are some cases where the object we find 
            # does not have the absolute_url method.
            cont = 'implements_container'
            publi = 'implements_publishable'
	    if not (hasattr(obj,cont) and hasattr(obj,publi)):
               preview_mode = ''
            elif not (getattr(obj,cont)() or getattr(obj,publi)()):
               preview_mode = ''
            return obj.absolute_url() + preview_mode
        # In all other cases:
        return path

InitializeClass(ParagraphSupport)

class HeadingSupport(ParagraphSupport):
    """ Mixed content support for SilvaDocument headings
    """
    
    security = ClassSecurityInfo()

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'parse')
    def parse(self, inputstr):
        """ Parse input string to a DOM
        """
        inputstr = unicode(inputstr, 'UTF-8')
        node = self._node
        # remove all old subnodes of node
        while node.hasChildNodes():
            node.removeChild(node.firstChild)
        HEADING_SUBSET.filteredParse(inputstr, node)

InitializeClass(HeadingSupport)            

class FormLabelSupport(HeadingSupport):
    '''Overloads SilvaDocument.ParagraphSupport to provide markup support
    for html field labels'''

    security = ClassSecurityInfo()

    security.declareProtected(SilvaPermissions.ChangeSilvaContent, 'parse')
    def parse(self, inputstr):
        """ Parse input string to a DOM
        """
        inputstr = unicode(inputstr, 'UTF-8')
        HEADING_SUBSET.filteredParse(inputstr, node)

InitializeClass(FormLabelSupport)
